@@ -209,50 +209,58 @@ guarana_intrp_reify_method_invocation(struct Hjava_lang_Object *obj, Method *met
     NativeResult *res;
     va_list arg;
     char *sig;
+    int argcount;
 
     op = guarana_new_native_operation(obj, CURRENTTHREAD(), 0);
-    op->kind = op_method_invocation;
+    if (equalUtf8Consts(method->name, init_name) ||
+	equalUtf8Consts(method->name, constructor_name))
+      op->kind = op_constructor_invocation;
+    else
+      op->kind = op_method_invocation;
     op->op_id.method = method;
     op->op_arg.args = ALLOC_ARGLIST_FOR(METHOD_SIGNATURE(method));
 
     arg = ARGLIST_TO_VA_LIST(METHOD_SIGNATURE(method), op->op_arg.args);
+    sig = op->op_id.method->signature->data;
+    argcount = sizeofSig(&sig, true);
+    args += argcount;
 
     for(sig = op->op_id.method->signature->data+1;
-	*sig != ')'; ++sig, ++args) {
+	*sig != ')'; ++sig) {
       switch(*sig) {
       case 'Z':
-	VA_ADD_ARG(arg, jboolean, args->v.tint);
+	VA_ADD_ARG(arg, jboolean, (--args)->v.tint);
 	break;
       case 'B':
-	VA_ADD_ARG(arg, jbyte, args->v.tint);
+	VA_ADD_ARG(arg, jbyte, (--args)->v.tint);
 	break;
       case 'S':
-	VA_ADD_ARG(arg, jshort, args->v.tint);
+	VA_ADD_ARG(arg, jshort, (--args)->v.tint);
 	break;
       case 'I':
-	VA_ADD_ARG(arg, jint, args->v.tint);
+	VA_ADD_ARG(arg, jint, (--args)->v.tint);
 	break;
       case 'J':
-	VA_ADD_ARG(arg, jlong, args->v.tlong);
+	VA_ADD_ARG(arg, jlong, (args-=2)->v.tlong);
 	++args;
 	break;
       case 'C':
-	VA_ADD_ARG(arg, jchar, args->v.tint);
+	VA_ADD_ARG(arg, jchar, (--args)->v.tint);
 	break;
       case 'F':
-	VA_ADD_ARG(arg, jfloat, args->v.tfloat);
+	VA_ADD_ARG(arg, jfloat, (--args)->v.tfloat);
 	break;
       case 'D':
-	VA_ADD_ARG(arg, jdouble, args->v.tdouble);
+	VA_ADD_ARG(arg, jdouble, (args-=2)->v.tdouble);
 	++args;
 	break;
       case 'L':
-	VA_ADD_ARG(arg, jobject, args->v.taddr);
+	VA_ADD_ARG(arg, jobject, (--args)->v.taddr);
 	while(*++sig != ';')
 	  ;
 	break;
       case '[':
-	va_arg(arg, jobject) = args->v.taddr;
+	va_arg(arg, jobject) = (--args)->v.taddr;
 	while(*++sig == '[')
 	  ;
 	if (*sig == 'L')
@@ -314,7 +322,11 @@ guarana_reify_method_invocation(struct HBR_unicamp_Guarana_MetaObject *mo, struc
   char *sig;
 
   op = guarana_new_native_operation(obj, CURRENTTHREAD(), 0);
-  op->kind = op_method_invocation;
+  if (equalUtf8Consts(method->name, init_name) ||
+      equalUtf8Consts(method->name, constructor_name))
+    op->kind = op_constructor_invocation;
+  else
+    op->kind = op_method_invocation;
   op->op_id.method = method;
   op->op_arg.args = ALLOC_ARGLIST_FOR(METHOD_SIGNATURE(method));
   COPY_ARGLIST_FROM_TO(METHOD_SIGNATURE(method), VA_LIST_TO_ARGLIST(METHOD_SIGNATURE(method), args), op->op_arg.args);
@@ -713,7 +725,7 @@ guarana_meta_init(struct Hjava_lang_Object *obj, struct Hjava_lang_Class *cc, st
 
   if (creator && creator->meta_object) {
       jobject newMetaObject = (jobject)do_execute_java_method(0, (jobject)creator->meta_object, "configure", "(Ljava/lang/Object;Ljava/lang/Object;)LBR/unicamp/Guarana/MetaObject;", 0, 0, obj, creator);
-    do_execute_java_class_method("BR/unicamp/Guarana/Guarana", "setMetaObjectOf", "(Ljava/lang/Object;LBR/unicamp/Guarana/MetaObject;LBR/unicamp/Guarana/MetaObject;)V", obj, (jobject)obj->meta_object, newMetaObject);
+      do_execute_java_class_method("BR/unicamp/Guarana/Guarana", "setMetaObjectOf", "(Ljava/lang/Object;LBR/unicamp/Guarana/MetaObject;LBR/unicamp/Guarana/MetaObject;)V", obj, (jobject)obj->meta_object, newMetaObject);
   }
 
   if (mo) {
@@ -736,7 +748,7 @@ guarana_meta_init(struct Hjava_lang_Object *obj, struct Hjava_lang_Class *cc, st
     assert(newObjConstr != 0);
 
     newObjMsg = newObjectNoMessage(newObjMsgClass);
-    do_execute_java_method(0, newObjMsg, 0, 0, newObjConstr, 0, newObjMsg);
+    do_execute_java_method(0, newObjMsg, 0, 0, newObjConstr, 0, obj);
     
     do_execute_java_class_method("BR/unicamp/Guarana/Guarana", "broadcast", "(LBR/unicamp/Guarana/Message;Ljava/lang/Object;)V", newObjMsg, (jobject)OBJECT_CLASS(obj));
   }
